from zope.component import getUtility, getMultiAdapter
from zope.site.hooks import setHooks, setSite
from zope.component import getUtility, getMultiAdapter

from zope.interface import directlyProvides

from Products.GenericSetup.utils import _getDottedName

from plone.portlets.interfaces import IPortletType
from plone.portlets.interfaces import IPortletManager
from plone.portlets.interfaces import IPortletAssignment
from plone.portlets.interfaces import IPortletDataProvider
from plone.portlets.interfaces import IPortletRenderer

from on.video.browser import featuredportlet
from on.video.testing import ON_VIDEO_FUNCTIONAL_TESTING
from plone.app.portlets.storage import PortletAssignmentMapping
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

from plone.app.layout.navigation.interfaces import INavigationRoot

import unittest2 as unittest

class TestPortlet(unittest.TestCase):

    layer = ON_VIDEO_FUNCTIONAL_TESTING

    def setUp(self):
        setHooks()
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def testPortletTypeRegistered(self):
        portlet = getUtility(IPortletType, name='irill.FeaturedVideos')
        self.assertEquals(portlet.addview, 'irill.FeaturedVideos')

    def testInterfaces(self):
        portlet = featuredportlet.Assignment()
        self.failUnless(IPortletAssignment.providedBy(portlet))
        self.failUnless(IPortletDataProvider.providedBy(portlet.data))

    def testInvokeAddview(self):
        portlet = getUtility(IPortletType, name='irill.FeaturedVideos')
        mapping = self.portal.restrictedTraverse('++contextportlets++plone.leftcolumn')
        for m in mapping.keys():
            del mapping[m]
        addview = mapping.restrictedTraverse('+/' + portlet.addview)

        addview.createAndAdd(data={})

        self.assertEquals(len(mapping), 1)
        self.failUnless(isinstance(mapping.values()[0], featuredportlet.Assignment))

    def testInvokeEditView(self):
        mapping = PortletAssignmentMapping()
        request = self.portal.REQUEST

        mapping['foo'] = featuredportlet.Assignment(header='test portlet', entries = 5)
        editview = getMultiAdapter((mapping['foo'], request), name='edit')
        self.failUnless(isinstance(editview, featuredportlet.EditForm))

    def testRenderer(self):
        context = self.portal
        request = context.REQUEST
        view = context.restrictedTraverse('@@plone')
        manager = getUtility(IPortletManager, name='plone.leftcolumn', context=self.portal)
        assignment = featuredportlet.Assignment()

        renderer = getMultiAdapter((context, request, view, manager, assignment), IPortletRenderer)
        self.failUnless(isinstance(renderer, featuredportlet.Renderer))



# schema etc. for videos

# (c) 2012-2013 oeko.net
# c/o toni mueller <support@oeko.net>
# license: GPLv3


import logging
from five import grok
from zope import schema
from zope.interface import Interface, Invalid
from urlparse import urljoin
import string

from plone.directives import form
from Products.CMFCore.utils import getToolByName

from plone.memoize.instance import memoize
from plone.app.textfield import RichText

from plone.app.layout.globals.interfaces import IViewView
from plone.app.layout.viewlets.interfaces import IBelowContent
from plone.app.layout.viewlets.interfaces import IBelowContentBody

from Products.Archetypes.interfaces.base import IBaseContent

from datetime import datetime

from on.video import _

class IVideo(form.Schema):
    """A video metadata object.
    """

### disable until we know how to deal with the resulting errors:
#    id = schema.TextLine(
#        title=_(u"Id"),
#        required=False,
#        )
    
    title = schema.TextLine(
        title=_(u"Name"),
        )
    
    director = schema.TextLine(
        title=_(u"Author"),
        required=False,
        )

    recorded = schema.Datetime(
        title=_(u"Date of recording"),
        required=False,
        )
    
    place = schema.Text(
        title=_(u"Location of recording"),
        required=False,
        )

    filename = schema.TextLine(
        title=_(u"Basename of the video metadata file - eg. for 'myvideo.metadata', enter 'myvideo'."),
        )

    description = schema.Text(
        title=_(u"Teaser"),
        description=_(u"Please put in a short summary about the "
                      "video in here. No formatting allowed."),
        required=False
        )
    
    body = RichText(
        title=_(u"Long description (allows some HTML)"),
        )



import os.path
import re
from Acquisition import aq_inner
from zope.component import queryUtility
from plone.registry.interfaces import IRegistry

from on.video.configuration import IVideoConfiguration

filetypes = {'mp4': 'video/mp4',
             'mpeg': 'video/mpeg',
             'flv': 'application/x-shockwave-flash',
             'ogv': 'video/ogg',
             'webm': 'video/webm',
             'avi': 'video/x-msvideo',
             'mov': 'video/quicktime',
             }


class vVideo(object):
    """Represent one (url, formatspec) pair for a given video.
       Used to build the list of downloadable Videos.
    """
    def __init__(self, url, formatspec):
        self.url = url
        self.displayformat = formatspec
        self.filetype = self.calculateFileType(url)

    @classmethod
    def calculateFileType(cls, name):
        """Do the windows thing and 'determine' the file type from the extension."""
        name = name.lower()
        slashpos = name.rfind('/')
        if slashpos >= 0:
            lastpart = name[slashpos + 1:]
        else:
            lastpart = name
        dotpos = lastpart.rfind('.')
        if dotpos >= 0:
            extension = lastpart[dotpos + 1:]
        else:
            extension = ''
        if extension in filetypes.keys():
            ftype = filetypes[extension]
        else:
            ftype = 'application/octet-stream'
        return ftype


dl_vtypes = ('video/ogg', 'video/webm', 'video/mp4', 'video/quicktime',
                 'video/x-msvideo', 'application/x-shockwave-flash',
                 'application/octet-stream')
player_vtypes = ('video/mp4', 'video/ogg', 'video/webm',
                     'video/quicktime', 'video/x-msvideo',
                     'application/x-shockwave-flash',
                     'application/octet-stream')


def sortVideosList(videos, desiredsorting):
    """Sort the videos according to the list of extensions in
       'desiredsorting'. No special sorting algorithm used,
       due to the estimated small size of the list.
    """
    result = []
    #print ">>> sortVideosList: videos = ", [ v.url for v in videos ]
    for ext in desiredsorting:
        #print "=== sortVideosList: videos = ", [ v.url for v in videos ]
        video = 0
        while len(videos) > 0 and video < len(videos):
            #print "--- checking video: ", video
            if videos[video].filetype == ext:
                result.append(videos[video])
                videos.pop(video)
                #print "--- appended ", result[-1].url
            else:
                video += 1
    #print "<<< sortVideosList: result = ", [ v.url for v in result ]
    return result

def sortVideosForDownload(videos):
    """Sort the videos for download, according to freeness and
       features. The 'videos' parameter is a list of vVideo
       objects.
    """
    return sortVideosList(videos, dl_vtypes)

def sortVideosForPlayer(videos, selected):
    """Sort the videos for download, according to freeness and
       features. The 'videos' parameter is a list of vVideo
       objects. Try to get the selected video into the first
       position, but only if it is an MP4.
    """
    #print ">>> sortVideosForPlayer(%s, %s)" % (str([ v.url for v in videos]), selected)
    result = sortVideosList(videos, player_vtypes)
    #print "--- sortVideosForPlayer(): result = ", [ v.url for v in result]
    if selected and (selected.endswith('mp4') or selected.endswith('MP4')):
        pos = 0
        for video in result:
            if video.url.endswith(selected):
                break
            pos += 1
        if pos > 0:
            swap = result[pos]
            result[pos] = result[0]
            result[0] = swap
    return result


### Reading the metadata file

import config

def setDefaultNoVideoValues(view):
    from config import DEFAULT_WIDTH
    from config import DEFAULT_HEIGHT
    view.thumbnailurl = '/++resource++on.video/novideo.png'
    view.playing_time = 'unknown' # '00:00:00'
    view.videos = []
    view.playfiles = []
    # special case: metafile is present, but not the actual video
    if not hasattr(view, "y") or not isinstance(view.x, int):
	view.x = DEFAULT_WIDTH
    if not hasattr(view, "y") or not isinstance(view.y, int):
	view.y = DEFAULT_HEIGHT


def fixupConfig():
    """Fix up the configuration, if required."""
    registry = queryUtility(IRegistry)
    settings = registry.forInterface(IVideoConfiguration)
    if settings.urlbase[-1] != '/':
        settings.urlbase = settings.urlbase + '/'
    if not (os.path.exists(settings.fspath) and os.path.isdir(settings.fspath)):
        logger = logging.getLogger('on.video')
        logging.warn("The configured path '%s' is not a directory, setting it to /tmp" % settings.fspath)
        settings.fspath = u'/tmp'        # fake it...
    return settings


def genUrl(prefix, folderspec, filename):
    """Concatenate these three elements and make them a real URL.
       No luck with urljoin(), so do it by hand for now.
    """
    tail = filename
    if folderspec:
        tail = folderspec + '/' + filename
    return prefix + tail
        

### Reading the metadata files:

class O(object):
    pass


def removeMetadataCommentLines(lines):
    """Remove all empty lines and comment lines from the list"""
    return [ l.strip() for l in lines if not (l.startswith("\n") or l.startswith("#")) ]


def genAbsolutePathToMetaFile(d, f):
    """d: directory, f: basename"""
    return os.path.join(d, f) + ".metadata"


def getMetaDataFileLines(fspath, filename):
    """Read the metadata file, so this part can be factored out from
       the other functions, improving testability. The function attempts
       to read no more than 2000 bytes (typical files tend to be like
       250-400 bytes long).
    """
    result = []
    lines = []
    meta_path = genAbsolutePathToMetaFile(fspath, filename)
    if os.path.exists(meta_path):
        fh = open(meta_path, "rb")
        if fh:
            lines = fh.readlines(2000)        # artificial limit... :|
            lines = removeMetadataCommentLines(lines)
            fh.close()
    for line in lines:
        if ':' in line:
            result.append(map(string.strip, line.split(':', 1)))
    return result


def parseMetadataFileContents(lines, urlbase, fspath, filename, vo = O(), player = config.PLAYER):
    """Parse the contents of the metadata file, passed in as a list
       of lines via the 'lines' argument, and set attributes on the
       passed-in object, which is assumed to be a view (but should
       work with a generic object).

       'lines' must be a list of lines from the metadata file
    """
    from config import MAX_WIDTH
    from config import DEFAULT_WIDTH
    from config import MIN_HEIGHT
    from config import MAX_HEIGHT
    from config import DEFAULT_HEIGHT
    
    vo.urlprefix = ""

    ### context = video object, context.filename = relative path to the metadata
    if '/' in filename:
        vo.urlprefix = filename[:filename.rfind('/')]

    meta_path = genAbsolutePathToMetaFile(fspath, filename)
    if not os.path.exists(meta_path):
        #print "no metadata for ", context
        setDefaultNoVideoValues(vo)
        vo.thumbnailurl = '/++resource++on.video/nometafile.png'
        return vo

    vo.thumbnailurl = None
    #import pdb; pdb.set_trace()
    try:
        line = lines.pop(0)
        if line[0] == 'thumbnail' and len(line) > 1 and line[1] != '':
            vo.thumbnailurl = genUrl(urlbase, vo.urlprefix, line[1])
        line = lines.pop(0)
        if line[0] == 'playing time' and len(line) > 1:
            pt = re.search('(\d+:\d\d:\d\d)', line[1])
            if pt:
                vo.playing_time = pt.group()
            else:
                vo.playing_time = 'unknown' #None #'0:00:00' # unnown playing time
        else:
            lines.insert(0, line)           # preserve the unused line

        vo.player = urlbase + player
        # playing time:
        # set url to the video that should be played inline (how to manage different sizes?):
        line = lines.pop(0)

        # Algorithm:
        # We need to select an MP4 format video for direct play, if possible.
        # To arrive at a consistent list of video files to play/download, we
        # only remember the file name of the selected video and see, whether
        # it occurs again later down the road (eg. should there be multiple
        # MP4 videos available).
        vo.directplay = None
        directplay = None
        #print "*** readVideoMetaData(), urlprefix = ", vo.urlprefix
        if line[0] == 'selected':
            vf = None
            if len(line) > 1:
                vf = line[1]
                if len(vf) > 1 and vf[0] == '/':
                    vf = vf[1:]
            if len(vf) and os.path.exists(os.path.join(fspath, vo.urlprefix, vf)):
                directplay = vf

        vo.x = DEFAULT_WIDTH
        vo.y = DEFAULT_HEIGHT
        if 'default size' in lines[0]:          # skip if we don't have it, but don't eat the line
            line = lines.pop(0)
            #print "*** reading the specified default size: ", dimensions
            x, y = map(int, line[1].split('x', 1))
            #print "\t\tx = %d, y = %d" % (x, y)
            # make sure the video doesn't get too small or too big:
            if x < 100 or x > MAX_WIDTH:
                #print "need to adjust the default size (x)"
                x = DEFAULT_WIDTH
            if y < MIN_HEIGHT or y > MAX_HEIGHT:
                #print "need to adjust the default size (y)"
                y = x * DEFAULT_HEIGHT/DEFAULT_WIDTH
            #print "\tafter sanitizing values: x = %d, y = %d" % (x, y)
            vo.x = x
            vo.y = y
    except:
        setDefaultNoVideoValues(vo)
        vo.thumbnailurl = '/++resource++on.video/invalidmetafile.png'
        return vo
    videos = {}

    # now process the list of video files:
    for line in lines:
        #print "... readVideoMetaData(): current row: -->%s<--" % row
        if len(line) == 1:
            break
        # k: format, v: filename
        k, v = line[:]
        v = v
        #print "checking filenames: ", os.path.join(fspath, vo.urlprefix, v)
        if not k in videos.keys() and os.path.exists(os.path.join(fspath, vo.urlprefix, v)):
            v_url = genUrl(urlbase, vo.urlprefix, v)
            videos[k] = vVideo(v_url, k)
            #print "--- readVideoMetaData(): videos[%s] = %s" % (str(k), str(videos[k]))
    vlist = videos.values()
    #print "\tvideo dimensions before handling files: x = %d, y = %d" % (vo.x, vo.y)
    vo.playfiles = sortVideosForPlayer(vlist, directplay)
    #print "*** readVideoMetaData(): videos for player, types: ", [ r.filetype for r in vo.playfiles ]
    #vo.directplay = vo.playfiles[0]
    if len(vo.playfiles) == 0:
        setDefaultNoVideoValues(vo)
    else:
        vo.directplay = vo.playfiles[0]
    # deep copy!!!
    downloadlist = vo.playfiles[:]
    vo.videos = sortVideosForDownload(downloadlist)
    #print "*** readVideoMetaData() done, video dimensions: x = %d, y = %d" % (vo.x, vo.y)
    return vo


def setVideoMetaData(view, context):
    """Set the attributes related to the current video from the metadata
       file.
    """
    settings = fixupConfig()
    lines = getMetaDataFileLines(settings.fspath, context.filename)
    vo = parseMetadataFileContents(lines, settings.urlbase,
                                   settings.fspath, context.filename)
    for a in vo.__dict__:
        view.__setattr__(a, vo.__getattribute__(a))
    view._title = context.title
    return view


class ViewThumbnail(grok.View):
    grok.context(IVideo)
    grok.require('zope2.View')
    grok.name('summary')

    """View to display the video within an album view."""

    def __init__(self, context, request):
        super(ViewThumbnail, self).__init__(context, request)
        self.id = context.id
        setVideoMetaData(self, context)

    @memoize
    def thumbnail(self):
        """Calculate the URL to the thumbnail"""
        if self.thumbnailurl is not None:
            return self.thumbnailurl;
        else:
            return '/++resource++on.video/nothumbnail.png'

    @memoize
    def title(self):
        """Return a part of the title, suitable for a gallery view."""
        return self.context.title[:20]


class View(grok.View):
    grok.context(IVideo)
    grok.require('zope2.View')
    grok.name('view')

    """Basic Video View"""

    def __init__(self, context, request):
        super(View, self).__init__(context, request)
        setVideoMetaData(self, context)

    @memoize
    def videofiles(self):
        """Return a list of video urls for download"""
        return self.videos

    @memoize
    def playerchoices(self):
        """Return a list of video urls for the player"""
        #print "=== View.playerchoices(): videos for player, types: ", [ r.filetype for r in self.playfiles ]
        return self.playfiles

    @memoize
    def thumbnail(self):
        """Calculate the URL to the thumbnail"""
        if self.thumbnailurl is not None:
            return self.thumbnailurl;
        else:
            return '/++resource++on.video/nothumbnail.png'

    @memoize
    def x(self):
        return self.x

    @memoize
    def y(self):
        return self.y

    @memoize
    def genFlashVars(self):
        """Generate a correct config string for the flash player.

           Sample:  value="config={'playlist':['/videosGHM/Joakim_Verona-Emacs_XWidget-2-1.jpg'
                      ,{'url':'/videosGHM/Joakim_Verona-Emacs_XWidget.mp4','autoPlay':false}]}"

           (all in one line, without spaces)
        """
        thumbnail = self.thumbnail()
        try:
            video = self.directplay.url
        except:
            video = "video not available"
        config="config={'playlist':['%s',{'url':'%s','autoPlay':false}]}" % \
                (thumbnail, video)
        return config



# As per issue #469, bail out if the metadata file does not exist:

@form.validator(field=IVideo['filename'])
def validateFilename(value):
    """Raise an exception if the file does not exist, or is unaccessible."""
    registry = queryUtility(IRegistry)
    settings = registry.forInterface(IVideoConfiguration)
    meta_path = genAbsolutePathToMetaFile(settings.fspath, value)
    accessible = True
    try:
        fp = open(meta_path, "r")
        fp.close()
    except:
        accessible = False
    if not os.path.isfile(meta_path) or not accessible:
        raise Invalid(u"Video metadata file does not exist, or is inaccessible, at %s" % \
                      meta_path)
    settings = fixupConfig()
    lines = getMetaDataFileLines(settings.fspath, value)
    vo = parseMetadataFileContents(lines, settings.urlbase,
                                   settings.fspath, value)
    #if vo.thumbnailurl == '/++resource++on.video/novideo.png':
    #    raise Invalid(u"Corrupt video metadata file at %s" % meta_path)
    if '/++resource++on.video/' in vo.thumbnailurl and \
           vo.thumbnailurl != '/++resource++on.video/nothumbnail.png':
        raise Invalid(u"Corrupt video metadata file at %s" % meta_path)


@form.validator(field=IVideo['recorded'])
def validateRecorded(value):
    """Raise an exception if the recording date lies in the future."""
    if value is not None and value > datetime.now():
        raise Invalid(u"The video could not have been recorded in the future.")

#        raise schema.ValidationError(u"The video could not have been recorded in the future.")




from Acquisition import aq_inner

from zope import schema
from zope.interface import implements
from zope.formlib import form
from zope.component import getMultiAdapter
from zope.app.component.hooks import getSite

from plone.app.portlets.portlets import base
from plone.memoize.instance import memoize
from plone.portlets.interfaces import IPortletDataProvider
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from on.video import _
from on.video.video import IVideo, ViewThumbnail

class IFeaturedVideos(IPortletDataProvider):

    header = schema.TextLine(title=u'Portlet Title',
                            description=u'Title of the rendered Portlet',
                            max_length=40, 
                            required=True)

    entries = schema.Int(title=_(u'Number of Videos to display'),
                       description=_(u'How many items to list.'),
                       required=True,
                       default=3)


class Assignment(base.Assignment):
    implements(IFeaturedVideos)

    # header = _(u'x', default=u"Best Videos")
    # entries = 3
    
    def __init__(self, header=u"", entries=3):
        self.header = header
        self.entries = entries

    @property
    def title(self):
        return self.header


class AddForm(base.AddForm):
    form_fields = form.Fields(IFeaturedVideos)
    label = u"Add Portlet for Featured Videos"
    description = u"Configure the portlet."

    def create(self, data):
        return Assignment(**data)

class EditForm(base.EditForm):
    form_fields = form.Fields(IFeaturedVideos)
    label = u"Edit Portlet for Featured Videos"
    description = u"Configure the portlet."


class Renderer(base.Renderer):
    _template = ViewPageTemplateFile('featuredvideos.pt')

    def __init__(self, *args):
        base.Renderer.__init__(self, *args)

        context = aq_inner(self.context)
        plone_tools = getMultiAdapter((context, self.request), name=u'plone_tools')
        self.cat = plone_tools.catalog()
        
    def render(self):
        return self._template()

    @property
    def available(self):
        """Show the portlet only if there are one or more elements."""
        if len(self._data()) > 0:
               return True
        return False

    def viewthumb(self):
        rlist = []
        res = self._data()
        # import pdb; pdb.set_trace()
        for i in res:
            rdict = {'url': i.getURL()}
            obj = i.getObject()
            vth = ViewThumbnail(obj, obj)            
            rdict['thumbnail'] = vth.thumbnail()
            rdict['title'] = vth.title()
            rlist.append(rdict)

        return rlist 

    def featuredvideos(self):
        return self._data()

    @memoize
    def _data(self):
        # import pdb; pdb.set_trace()
        limit = self.data.entries
        results = self.cat(object_provides=IVideo.__identifier__,sort_on='effective',sort_order='ascending')[:limit]
        # print "RESULTS: " + str(results)
        return results

        



### copyright (c) 2012-2013 Toni Mueller <support@oeko.net>
### AGPLv3 if possible, otherwise GPLv3 or later
### incorporates portions of code from ore.contentmirror

"""
Usage:

./bin/instance run addvideos PORTAL_PATH FS_PATH


PORTAL_PATH: root folder for a Plone site in the ZODB (eg. /01/mnt/Plone)

FS_PATH: path where existing video files (.metadata) reside

The configured root path for all videos should be a prefix of FS_PATH,
otherwise your results will be garbage.

The target Plone site must have the on.video product installed,
otherwise this script will not work.

"""

import os
import os.path
import re
import sys
import optparse


try:
    from cStringIO import StringIO
except:
    from StringIO import StringIO

from datetime import datetime

import transaction

from Products.Five import zcml
from zope.component import queryMultiAdapter
from zope.component import queryUtility
from zope.app.component.hooks import setSite
from zope.i18n.interfaces import ITranslationDomain
from zope.i18n.locales import locales
from zope.interface import implements
from zope.app.component.hooks import getSite
from Products.CMFCore.exceptions import BadRequest
from plone.uuid.interfaces import IMutableUUID
from Products.CMFCore.utils import getToolByName
from Products.ATContentTypes.utils import dt2DT

from Products.CMFCore.utils import getToolByName

from AccessControl import getSecurityManager
from AccessControl import Unauthorized
#from AccessControl.Owner import changeOwnership
from Products.CMFCore import permissions
from plone.app.textfield.value import RichTextValue

from plone.registry.interfaces import IRegistry
from on.video.configuration import IVideoConfiguration

#from zopestuff import recreateSite

#from config import dburl, targetpath


site = None                             # the site we are working on

catalog = None

stdlicense = """
<a class="video-copyright-link" title="CC-BY-NC-ND" href="http://creativecommons.org/licenses/by-nc-nd/3.0/fr/">
<img title="Creative Commons BY-NC-ND" sclass="video-copyright-image" rc="resolveuid/525e886f48314790b6b83b3446ab4fb5" alt="Creative Commons BY-NC-ND">
</a>
"""

licenses = {'CC-BY-NC-ND': stdlicense }


def beautifyId(s):
    """Turn a string into something resembling a headline.
       Method: Split on all spaces and underscores, then capitalize
               every piece, then return the joined parts.
    """
    print "beautifyId(%s)" % s
    shards = [ c.capitalize() for c in re.split('[-_ ]', s) ]
    print "  crumbs = ", shards
    return " ".join(shards)


def getVideoList(path):
    """Get all videos in the specified directory"""
    registry = queryUtility(IRegistry)
    settings = registry.forInterface(IVideoConfiguration)
    vroot = settings.fspath
    prefix = ""
    if path.startswith(vroot) and path != vroot:
        prefix = path[len(vroot):]
        prefix = prefix.strip('/')
    allfiles = [ f[:f.find('.metadata')] for f in os.listdir(path) if f.endswith('.metadata') and f != '.metadata' ]
    result = {}
    for element in allfiles:
        result[os.path.join(prefix, element)] = beautifyId(element)
    return result

def addAllVideos(site, folder, fs_path, rights, options):
    """Add all videos found in the fs_path to the site."""
    parts = folder.split('/')
    target = site

    if options.publish:
        wft = getToolByName(site, 'portal_workflow')
    for part in parts:
        if part not in target.keys():
            target.invokeFactory('Folder', id=part, title=part)
        target = target[part]
    # issue #490:
    if options.layout:
        target.setLayout(options.layout)
    else:
        target.setLayout('videogallery')
    videos = getVideoList(fs_path)      # a dictionary
    for video in videos:
        newid = video[video.rfind('/')+1:]
        nv_id = target.invokeFactory('on.video.Video', id=newid, title=videos[video], filename = video)
        newvideo = target[nv_id]
        if rights:
            newvideo.body = RichTextValue('<p class="licenseimage">' + rights + '</p>')
        newvideo.setExcludeFromNav(True)
        if options.publish:
            wft.doActionFor(newvideo, 'publish')

### system integration boilerplate:

def get_app():
    frame = sys._getframe(2)
    return frame.f_locals.get('app')


def setup_parser():
    parser = optparse.OptionParser(
        usage="usage: ./bin/instance run ./bin/%prog [options] portal_path filesystem_path")
    parser.add_option(
        '-f', '--folder', dest="folder", default="",
        help="Which folder to import into (default: use an auto-generated name)")
    parser.add_option(
        '-l', '--license', dest="licensekey", default="CC-BY-NC-ND", action='store',
        help="""License string (default: CC-BY-NC-ND). If you want to use something else, specify the full required HTML code. If you want no license, specify NONE.""")
    parser.add_option(
        '-L', '--Layout', dest="layout", default="videogallery", action='store',
        help="""License string (default: CC-BY-NC-ND). If you want to use something else, specify the full required HTML code. If you want no license, specify NONE.""")
    parser.add_option(
        '-p', '--publish', dest="publish", default=False, action='store_true',
        help="Immediately publish all videos (they are likely to not be elegantly named)")
    return parser


### MAIN:

def main(app=None, instance_path=None):
    parser = setup_parser()
    options, args = parser.parse_args()
    global site

    if len(args) != 2:
        parser.print_help()
        sys.exit(1)
        return

    if app is None:
        app = get_app()
    try:
        instance_path = args[0]
        fs_path = args[1]
    except:
        parser.print_help()
        sys.exit(2)

    #folder = "imported-videos-%s" % datetime.now().strftime("%s")
    folder = 'imported-videos'
    licensekey = 'CC-BY-NC-ND'

    if not os.path.isdir(fs_path):
        parser.print_help()
        sys.exit(1)

    try:
        portal = app.unrestrictedTraverse(instance_path)
    except:
        print "The specified Plone site does not seem to exist, bailing out."
        parser.print_help()
        sys.exit(1)

    setSite(portal)
    site = portal

    if options.licensekey in licenses:
        rights = licenses[options.licensekey]
    elif options.licensekey != "NONE":
        rights = options.licensekey
    else:
        rights = None

    addAllVideos(site, folder, fs_path, rights, options)

    transaction.commit()




# gallery view for folders containing videos

# (c) 2012-2013 oeko.net
# c/o toni mueller <support@oeko.net>
# license: GPLv3

"""
Folder view that enables the display of videos like in a photo
gallery.

These are the differences to the normal gallery view of folders:

 * Videos are displayed using their summary view. May be
   generalized later.

 * Contained folders are searched for sub-galleries, and a
   thumbnail and description about the collection is being
   displayed.

 * Next-Previous navigation is generally enabled.

 * If possible, creating sub-folders should inherit this gallery
   view from their parent folder, so that contained folders are
   automatically also galleries by default.


Open question: How to revert the display settings for folders
on uninstallation of the product?

"""

from zope.interface import Interface
from five import grok

from Products.CMFPlone.PloneBatch import Batch
from Products.CMFCore.utils import getToolByName
from plone.memoize.instance import memoize
from plone.app.contentlisting import catalog
from on.video import _

from on.video.video import ViewThumbnail


class FolderItems(object):
    """Collect attributes for folder entries."""
    def __init__(self, data, datatype, start):
        self.data = data
        self.datatype = datatype
        self.filetype = self.calculateFileType(url)
        self.start = start

def XXXXcountFolderItems(item):
    """item count to be shown in a folder / collection summary view
       Return a pair (folders, videos).
    """
    counts = { 'Folder': 0, 'on.video.Video': 0 }
    #import pdb; pdb.set_trace()

    if item.getPortalTypeName() == 'Folder':
        # import pdb; pdb.set_trace()
        try:
            flist = item.folderlistingFolderContents()
        except:
            obj = item._brain.getObject()
            flist = obj.folderlistingFolderContents()
    if item.getPortalTypeName() == 'Collection':
        catalog = getToolByName(self, 'portal_catalog')
        flist = self.context.results(batch=False)

    for item in flist:
        if not item.portal_type in ('Folder', 'on.video.Video'):
            continue
        counts[item.portal_type] += 1
        
    return (counts['Folder'], counts['on.video.Video'])


def countFolderItems(folder):
    """Filter the given list of folder contents for those elements
       that are intended to be shown in the video gallery, and
       count them.
       Return a pair (folders, videos).
    """
    folderlisting = folder.folderlistingFolderContents()
    counts = { 'Folder': 0, 'on.video.Video': 0 }
    for item in folderlisting:
        if not item.portal_type in ('Folder', 'on.video.Video'):
            continue
        counts[item.portal_type] += 1
        g = 0
        v = 0
        if item.portal_type == 'Folder':
            (g, v) = countFolderItems(item)
        counts['Folder'] += g
        counts['on.video.Video'] += v
    return (counts['Folder'], counts['on.video.Video'])

def shorttitle(title):
    """shorten titles for gallery view"""
    if len(title) > 50:
        tshort = title[:47] + "..."
    else: tshort = title
    tdict = {'short': tshort, 'long': title}
    return tdict


def genSmallView(item, request = None):
    """Turn a content item into a dictionary. We only need specific
       information, depending on type.
    """
    result = dict(portaltype=item.portal_type, id=item.id, title=item.title)
    result['banner'] = '/++resource++on.video/nothumbnail.png'
    # import pdb; pdb.set_trace()
    try: # Are we are in a collection? 
        result['path'] =item.getPath()
    except: # Or in a Folder?
        result['path'] =item.absolute_url()
    
    if item.portal_type == 'Folder':
        (folders, videos) = countFolderItems(item)
        result['sub_folder'] = folders
        result['sub_videos'] = videos
        # assumption: the image is an ArcheTypes image
        if 'bannerimage' in item.keys():
            result['thumb'] = '%s/bannerimage/image' % item.id
        titles = shorttitle(item.title)
        result['title'] = titles['short']
        result['longtitle'] = titles['long']

    elif item.portal_type == 'on.video.Video':
        result['sub_folder'] = None
        result['sub_videos'] = None
        # this information is not accessible from here:
        vtn = ViewThumbnail(item, request)
        result['playingtime'] = vtn.playing_time
        result['thumb'] = vtn.thumbnail()
        titles = shorttitle(item.title)
        result['title'] = titles['short']
        result['director'] = item.director or ""
        result['longtitle'] = titles['long']

    elif item.portal_type == 'Image':
        result['sub_folder'] = None
        result['sub_videos'] = None
        result['thumb'] = '%s/image' % item.id
        titles = shorttitle(item.title)
        result['title'] = titles['short']
        result['longtitle'] = titles['long']
    else:
        raise ValueError, "item %s is an object of an illegal type." % str(item)
    # print result
    return result


class VideoGallery(grok.View):
    """Default view, gallery style, for a video folder.
    """
    grok.context(Interface)
    grok.require('zope2.View')

    #@memoize
    def getFolderContents(self):
        """Filter the folder contents for those elements that are
           intended to be shown in the video gallery.
        """
        if self.context.portal_type == 'Folder':
            contents = [ item for item in self.context.folderlistingFolderContents() if \
                         item.portal_type in ('Folder', 'on.video.Video', 'Image')
                         and item is not None]
        if self.context.portal_type == 'Collection':
            """partly taken from plone.app.collection"""
            catalog = getToolByName(self, 'portal_catalog')
            results = self.context.results(batch=False)

            contents = [ item for item in results if \
                         item.portal_type in ('Folder', 'on.video.Video', 'Image')
                         and item is not None]
            #print "VideoGallery(%s) getFolderContens: contents = %s" % (str(self), str(contents))
        return contents

    #@memoize
    def update(self):
        """Called before rendering the template for this view.
        """
        fl = [ x for x in self.getFolderContents() if x.id != 'bannerimage' ]
        b_start = int(self.context.REQUEST.get('b_start', 0))
        #print "VideoGallery() update: b_start = %d, fl = %s" % (b_start, str(fl))
        self.contents = Batch([ genSmallView(item, self.request) for item in fl ], size=12, start=b_start)



# schema etc. for videos

# (c) 2012 oeko.net
# c/o toni mueller <support@oeko.net>
# license: GPLv3


import os, os.path

from five import grok
from zope import schema
from zope.interface import implements
from plone.directives import form, dexterity

from on.video import _

from on.video import config


class IVideoConfiguration(form.Schema):
    """Configuration for the whole package.
    """

    fspath = schema.TextLine(
        title=_(u"File System Path"),
        description=_(u"File system path to the directory where the video files are stored."),
        required = True,
        default = config.ON_VIDEO_FS_PATH,
        )

    urlbase = schema.URI(
        title=_(u"Base URL"),
        description=_(u"Base URL for generated links. The full URL will "
                      "look like urlbase + filename + filetype."),
        required = True,
        default = config.ON_VIDEO_URL,
        )

from plone.app.registry.browser import controlpanel

class VideoConfigurationEditForm(controlpanel.RegistryEditForm):

    schema = IVideoConfiguration
    label = _(u"Video Product settings")
    description = _(u"""""")

    def updateFields(self):
        super(VideoConfigurationEditForm, self).updateFields()


    def updateWidgets(self):
        super(VideoConfigurationEditForm, self).updateWidgets()

class VideoConfigurationControlPanel(controlpanel.ControlPanelFormWrapper):
    form = VideoConfigurationEditForm



# Testing on.video in Plone 4.1, using plone.app.testing:

# (c) 2012 oeko.net
# c/o toni mueller <support@oeko.net>
# license: GPLv3

import unittest2 as unittest
#from Products.CMFCore.utils import getToolByName

from plone.app.testing import PloneSandboxLayer
from plone.app.testing import quickInstallProduct
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import IntegrationTesting
from plone.app.testing import FunctionalTesting

import Products.CMFPlone

from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.app.testing import SITE_OWNER_NAME, SITE_OWNER_PASSWORD

#from plone.app.testing import setRoles

from plone.app.testing import applyProfile

from plone.testing import z2
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName

class OnVideoFixture(PloneSandboxLayer):
    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        import on.video
        self.loadZCML(package=on.video)

    def setUpPloneSite(self, portal):
        wftool = getToolByName(portal, 'portal_workflow')
        wftool.setDefaultChain('folder_workflow')
        """Run the GS profile for this product"""
        self.applyProfile(portal, 'on.video:default')

    def tearDownZope(self, app):
        """Uninstall the product and destroy the Zope site"""
        z2.uninstallProduct(app, 'on.video')

    def manager_browser(self):
        """Browser of Manager - not accessible from test layer
        """
        return self._auth_browser(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
    

ON_VIDEO_FIXTURE = OnVideoFixture()
ON_VIDEO_INTEGRATION_TESTING = IntegrationTesting(bases=(ON_VIDEO_FIXTURE,), name="OnVideoFixture:Integration")
ON_VIDEO_FUNCTIONAL_TESTING = FunctionalTesting(bases=(ON_VIDEO_FIXTURE,), name="OnVideoFixture:Functional")



# (c) 2012 oeko.net
# c/o toni mueller <support@oeko.net>
# license: GPLv3

from datetime import datetime

from zope.component import queryUtility

from plone.registry.interfaces import IRegistry
from plone.app.textfield.value import RichTextValue

from on.video.testing import ON_VIDEO_FUNCTIONAL_TESTING

from on.video.configuration import IVideoConfiguration
from on.video import config
import os
import tempfile
import shutil

import unittest2 as unittest

from plone.app.testing import TEST_USER_ID
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD

from plone.app.testing import setRoles

from on.video.testing import ON_VIDEO_FUNCTIONAL_TESTING
from plone.testing.z2 import Browser

class TestOnVideoHandling(unittest.TestCase):
    """Test the code for handling video objects, views etc."""

    layer = ON_VIDEO_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        registry = queryUtility(IRegistry)
        self.settings = registry.forInterface(IVideoConfiguration)
        # generate a set of mockup files:
        self.td = tempfile.mkdtemp(prefix = "on.video-tests.")
        videofiles = ['sample_video_1_big.ogv', 'sample_video_1_medium.ogv', 'sample_video_1.mp4',
                      'sample_video_2.mp4', 'sample_video_2.avi',
                      'sample_video_3.mp4', 'sample_video_3_medium.ogv',
                      ]
        files_to_copy = [ 'sample_video_%d.metadata' % i for i in range(1, 8) ]
        otherfiles = [ 'Vincent_Untz.metadata', 'corrupted1.metadata', 'corrupted2.metadata', 'thumb.png' ]
        files_to_copy = files_to_copy + otherfiles  + videofiles
        for f in files_to_copy:
            sampledata = os.path.join(os.path.dirname(__file__), f)
            shutil.copy(sampledata, self.td)
        subdir = os.path.join(self.td, "subfolder")
        os.mkdir(subdir)
        sampledata = os.path.join(os.path.dirname(__file__), 'sample_video_3.metadata')
        shutil.copy(sampledata, subdir)
        for d in self.td, subdir:
            for video in videofiles:
                o = open(os.path.join(d, video), "wb")
                o.write("1")
                o.close()
        self.settings.fspath = unicode(self.td)


    def tearDown(self):
        """remove the temp stuff"""
        shutil.rmtree(self.td)

    def test_create_video_object(self):
        """Create a video object and inspect its attributes to see whether
           it conforms to the specs.
        """
        v = self.portal.invokeFactory('on.video.Video', 'video1', title=u"My Sample Video")
        self.failUnless(v in self.portal)


    def test_read_video1_metadata(self):
        """Create a video object and inspect its attributes to see whether
           it conforms to the specs.
        """
        #import pdb; pdb.set_trace()
        v = self.portal.invokeFactory('on.video.Video', 'video1', title=u"My Sample Video",
                                      name = 'some kind of video',
                                      author = 'me, myself',
                                      recorded = datetime.now(),
                                      filename = 'sample_video_1',
                                      place = 'nirvana',
                                      body = RichTextValue('<strong>some interesting story</strong>'))
        import transaction; transaction.commit()
        app = self.layer['app']
        browser = Browser(app)
        browser.handleErrors = False
        video = self.portal[v]
        view = video.restrictedTraverse('@@view')
        downloads = view.videofiles()
        playlist = view.playerchoices()
        self.failUnless(playlist[0].url.endswith(".mp4"))
        self.failUnless(downloads[0].url.endswith(".ogv"))
        browser.open(video.absolute_url())
        self.failUnless('nothumbnail' in browser.contents)

    def test_read_video2_metadata(self):
        """Create a video object and inspect its attributes to see whether
           it conforms to the specs.
        """
        v = self.portal.invokeFactory('on.video.Video', 'video2', title=u"My Sample Video",
                                      name = 'some kind of video',
                                      author = 'me, myself',
                                      recorded = datetime.now(),
                                      filename = 'sample_video_2',
                                      place = 'nirvana',
                                      body = RichTextValue('<strong>some interesting story</strong>'))
        import transaction; transaction.commit()
        app = self.layer['app']
        browser = Browser(app)
        browser.handleErrors = False
        video = self.portal[v]
        browser.open(video.absolute_url())
        self.failUnless('nothumbnail' in browser.contents)

    def test_read_video3_metadata(self):
        """Create a video object and inspect its attributes to see whether
           it conforms to the specs.
        """
        v = self.portal.invokeFactory('on.video.Video', 'video2', title=u"My Sample Video",
                                      name = 'some kind of video',
                                      author = 'me, myself',
                                      recorded = datetime.now(),
                                      filename = 'Vincent_Untz',
                                      place = 'nirvana',
                                      body = RichTextValue('<strong>some interesting story</strong>'))
        import transaction; transaction.commit()
        app = self.layer['app']
        browser = Browser(app)
        browser.handleErrors = False
        video = self.portal[v]
        browser.open(video.absolute_url())
        #print "browser.contents: ", browser.contents
        self.failUnless('novideo' in browser.contents)

    def test_read_video_with_no_playing_time(self):
        """Check the generated playlist: What about unknown file extensions?
        """
        from on.video.config import DEFAULT_WIDTH, DEFAULT_HEIGHT
        v = self.portal.invokeFactory('on.video.Video', 'video8', title=u"My Sample Video",
                                      name = 'some kind of video',
                                      author = 'me, myself',
                                      recorded = datetime.now(),
                                      filename = 'sample_video_6',
                                      place = 'nirvana',
                                      body = RichTextValue('<strong>some interesting story</strong>'))
        video = self.portal[v]
        import transaction; transaction.commit()
        view = video.restrictedTraverse('@@view')
        import pdb; pdb.set_trace()
        self.failUnless(view.playing_time == 'unknown')

    def test_read_video_no_metadata(self):
        """Create a video object that has no metadata file.
           See whether the code detects this properly and
           displays the appropriate placeholder image.
        """
        v = self.portal.invokeFactory('on.video.Video', 'video3', title=u"My Sample Video",
                                      name = 'some kind of video',
                                      author = 'me, myself',
                                      recorded = datetime.now(),
                                      filename = 'no-metadata',
                                      place = 'nirvana',
                                      body = RichTextValue('<strong>some interesting story</strong>'))
        import transaction; transaction.commit()
        app = self.layer['app']
        browser = Browser(app)
        browser.handleErrors = False
        video = self.portal[v]
        browser.open(video.absolute_url())
        self.failUnless('nometafile' in browser.contents)

    def XXtest_add_video_via_browser_no_metadata(self):
        """Simulate adding a video object that has no metadata file.
           The validator should raise an exception, which should
           yield an error message in the browser.

           We first need to learn about properly writing browser tests.
        """
        v = self.portal.invokeFactory('on.video.Video', 'video',
                                      title=u"My Sample Unavailable Video",
                                      name = 'some kind of video',
                                      author = 'me, myself',
                                      recorded = datetime.now(),
                                      filename = 'no-metadata',
                                      place = 'nirvana',
                                      body = RichTextValue('<strong>some interesting story</strong>'))
        import transaction; transaction.commit()
        app = self.layer['app']
        browser = Browser(app)
        browser.handleErrors = False
        video = self.portal[v]
        browser.open(video.absolute_url())
        self.failUnless('nometafile' in browser.contents)

    def test_read_video_metadata_no_video(self):
        """Create a video object that has a metadata file that points to
           non-existent video files..
           See whether the code detects this properly and
           displays the appropriate placeholder image.
        """
        v = self.portal.invokeFactory('on.video.Video', 'video5', title=u"My Sample Video",
                                      name = 'some kind of video',
                                      author = 'me, myself',
                                      recorded = datetime.now(),
                                      filename = 'sample_video_4',
                                      place = 'nirvana',
                                      body = RichTextValue('<strong>some interesting story</strong>'))
        import transaction; transaction.commit()
        app = self.layer['app']
        browser = Browser(app)
        browser.handleErrors = False
        video = self.portal[v]
        browser.open(video.absolute_url())
        #print "browser.contents: ", browser.contents
        self.failUnless('novideo' in browser.contents)
        ### Disabled, because we so far inherently ignore unknown files:
        #self.failUnless('application/octet-stream' in browser.contents)

    def test_read_video_with_thumbnail(self):
        """Create a video object with a thumbnail.
        """
        v = self.portal.invokeFactory('on.video.Video', 'video4', title=u"My Sample Video",
                                      name = 'some kind of video',
                                      author = 'me, myself',
                                      recorded = datetime.now(),
                                      filename = 'sample_video_3',
                                      place = 'nirvana',
                                      body = RichTextValue('<strong>some interesting story</strong>'))
        video = self.portal[v]
        import transaction; transaction.commit()
        view = video.restrictedTraverse('@@view')
        downloads = view.videofiles()
        playlist = view.playerchoices()
        self.failUnless(view.playing_time == '20:30:50')
        self.failUnless(playlist[0].url.endswith(".mp4"))
        self.failUnless(downloads[0].url.endswith(".ogv"))

    def test_video_summary_view(self):
        """check whether the 'summary' view for the gallery works
        """
        v = self.portal.invokeFactory('on.video.Video', 'video4', title=u"My Sample Video",
                                      name = 'some kind of video',
                                      author = 'me, myself',
                                      recorded = datetime.now(),
                                      filename = 'sample_video_3',
                                      place = 'nirvana',
                                      body = RichTextValue('<strong>some interesting story</strong>'))
        video = self.portal[v]
        import transaction; transaction.commit()
        app = self.layer['app']
        browser = Browser(app)
        browser.handleErrors = False
        video = self.portal[v]
        browser.open(video.absolute_url() + '/@@summary')
        self.failUnless('<div class="on-video-small-thumbnail">' in browser.contents)
        self.failIf('<object>' in browser.contents)

    def test_video_summary_view(self):
        """check whether the 'summary' view for the gallery works
        """
        v = self.portal.invokeFactory('on.video.Video', 'video4', title=u"My Sample Video",
                                      name = 'some kind of video',
                                      author = 'me, myself',
                                      recorded = datetime.now(),
                                      filename = 'sample_video_3',
                                      place = 'nirvana',
                                      body = RichTextValue('<strong>some interesting story</strong>'))
        video = self.portal[v]
        import transaction; transaction.commit()
        app = self.layer['app']
        browser = Browser(app)
        browser.handleErrors = False
        video = self.portal[v]
        browser.open(video.absolute_url() + '/@@summary')
        self.failUnless('<div class="on-video-small-thumbnail">' in browser.contents)
        self.failIf('<object>' in browser.contents)

    def test_read_video_from_subdir(self):
        """See whether we can serve a video from a subdirectory.
           The hardcoded valeu for the URL prefix should be replaced
           by reading the registry.
        """
        v = self.portal.invokeFactory('on.video.Video', 'video5', title=u"My Sample Video",
                                      name = 'some kind of video',
                                      author = 'me, myself',
                                      recorded = datetime.now(),
                                      filename = 'subfolder/sample_video_3',
                                      place = 'nirvana',
                                      body = RichTextValue('<strong>some interesting story</strong>'))
        video = self.portal[v]
        import transaction; transaction.commit()
        view = video.restrictedTraverse('@@view')
        self.failUnless('http://localhost/subfolder/' in view.thumbnail())
        downloads = view.videofiles()
        for video in downloads:
            self.failUnless('http://localhost/subfolder/' in video.url)

    def test_read_video_default_dimensions_handling(self):
        """See whether the dimensions are set correctly if the metadata file
           does not include the "default size" entry.
        """
        from on.video.config import DEFAULT_WIDTH, DEFAULT_HEIGHT
        v = self.portal.invokeFactory('on.video.Video', 'video6', title=u"My Sample Video",
                                      name = 'some kind of video',
                                      author = 'me, myself',
                                      recorded = datetime.now(),
                                      filename = 'sample_video_3',
                                      place = 'nirvana',
                                      body = RichTextValue('<strong>some interesting story</strong>'))
        video = self.portal[v]
        import transaction; transaction.commit()
        view = video.restrictedTraverse('@@view')
        self.failUnless(view.x == DEFAULT_WIDTH)
        self.failUnless(view.y == DEFAULT_HEIGHT)

    def test_read_video_custom_dimensions_handling(self):
        """See whether the dimensions are set correctly if the metadata
	   file does include the "default size" entry.
        """
        from on.video.config import DEFAULT_WIDTH, DEFAULT_HEIGHT
        v = self.portal.invokeFactory('on.video.Video', 'video7', title=u"My Sample Video",
                                      name = 'some kind of video',
                                      author = 'me, myself',
                                      recorded = datetime.now(),
                                      filename = 'sample_video_4',
                                      place = 'nirvana',
                                      body = RichTextValue('<strong>some interesting story</strong>'))
        video = self.portal[v]
        import transaction; transaction.commit()
        view = video.restrictedTraverse('@@view')
        self.failUnless(view.x == 690)
        self.failUnless(view.y == 535)

    def test_read_video_weird_dimensions_handling(self):
        """See whether the dimensions are set correctly if the metadata
	   file does include the "default size" entry, but values are
	   out of bound.
        """
        from on.video.config import DEFAULT_WIDTH, DEFAULT_HEIGHT
        v = self.portal.invokeFactory('on.video.Video', 'video8', title=u"My Sample Video",
                                      name = 'some kind of video',
                                      author = 'me, myself',
                                      recorded = datetime.now(),
                                      filename = 'sample_video_5',
                                      place = 'nirvana',
                                      body = RichTextValue('<strong>some interesting story</strong>'))
        video = self.portal[v]
        import transaction; transaction.commit()
        view = video.restrictedTraverse('@@view')
        self.failUnless(view.x == DEFAULT_WIDTH)
        self.failUnless(view.y == DEFAULT_HEIGHT)

    def test_read_video_with_no_playing_time(self):
        """See whether the file is correctly read if the playing time
           is not specified.
        """
        from on.video.config import DEFAULT_WIDTH, DEFAULT_HEIGHT
        v = self.portal.invokeFactory('on.video.Video', 'video8', title=u"My Sample Video",
                                      name = 'some kind of video',
                                      author = 'me, myself',
                                      recorded = datetime.now(),
                                      filename = 'sample_video_6',
                                      place = 'nirvana',
                                      body = RichTextValue('<strong>some interesting story</strong>'))
        video = self.portal[v]
        import transaction; transaction.commit()
        view = video.restrictedTraverse('@@view')
        self.failUnless(view.playing_time == 'unknown')

    def test_fix_settings_url(self):
        """See whether the URL is properly fixed.
        """
        from on.video.video import fixupConfig
        oldsettings = self.settings
        self.settings.urlbase = 'http://nohost'
        fixupConfig()
        self.failUnless(self.settings.urlbase.endswith('/'))

    def test_fix_settings_fspath(self):
        """See whether the file system path is properly reset.
        """
        from on.video.video import fixupConfig
        oldsettings = self.settings
        self.settings.fspath = u'/nosuchfileordirectory'
        fixupConfig()
        self.failUnless(self.settings.fspath == u'/tmp')

    def test_video_format_thingy(self):
        from on.video.video import vVideo
        v = vVideo("bla.ogv", "OGV")
        self.failUnless(v.url == 'bla.ogv')
        self.failUnless(v.displayformat == 'OGV')
        self.failUnless(v.filetype == 'video/ogg')

    def test_rejection_of_corrupted_files(self):
        from on.video.video import parseMetadataFileContents, fixupConfig, getMetaDataFileLines
        settings = fixupConfig()
        lines = getMetaDataFileLines(settings.fspath, 'corrupted1')
        vo = parseMetadataFileContents(lines, settings.urlbase, settings.fspath, 'corrupted1')
        self.failUnless(vo.thumbnailurl == '/++resource++on.video/invalidmetafile.png')


    def test_add_video_without_title(self):
        """Fill out the video 'add' form with the wrong date and notice
           that it does not validate.
        """
        app = self.layer['app']
        browser = Browser(app)
        #browser.handleErrors = False
        # get the relevant fields:
        browser.addHeader('Authorization', 'Basic %s:%s' % (
            SITE_OWNER_NAME, SITE_OWNER_PASSWORD))
        browser.open(self.portal.absolute_url() + '/++add++on.video.Video')
        title_field = browser.getControl(name='form.widgets.title')
        recorded_field_year = browser.getControl(name='form.widgets.recorded-year')
        recorded_field_month = browser.getControl(name='form.widgets.recorded-month')
        recorded_field_day = browser.getControl(name='form.widgets.recorded-day')
        recorded_field_hour = browser.getControl(name='form.widgets.recorded-hour')
        recorded_field_min = browser.getControl(name='form.widgets.recorded-min')
        filename_field = browser.getControl(name='form.widgets.filename')
        body_field = browser.getControl(name='form.widgets.body')
        save_button = browser.getControl(name='form.buttons.save')
        # fill the fields and save the form:
        filename_field.value = u'sample_video_3'
        body_field.value = u'<p>nothing to see here, move along</p>'
        save_button.click()
        #import pdb; pdb.set_trace()
        vl = self.portal.keys()
        #self.failUnless(True)



# (c) 2012 oeko.net
# c/o toni mueller <support@oeko.net>
# license: GPLv3

from datetime import datetime

#from zope.testbrowser.browser import Browser
from plone.testing.z2 import Browser
#from zope.publisher.browser import TestRequest

from zope.component import queryUtility
from Products.CMFCore.utils import getToolByName

import transaction

from plone.registry.interfaces import IRegistry
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import login

from on.video.testing import ON_VIDEO_FUNCTIONAL_TESTING

from on.video.configuration import IVideoConfiguration
from on.video import config
import os
import tempfile
import shutil

import unittest2 as unittest

from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD

from plone.app.testing import setRoles

from on.video.testing import ON_VIDEO_FUNCTIONAL_TESTING



class TestVideoGallery(unittest.TestCase):
    """Test the code for handling video objects, views etc."""

    layer = ON_VIDEO_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        #setRoles(self.portal, TEST_USER_NAME, ['Manager'])
        self.portal.invokeFactory("Folder", id='testfolder', title='Testfolder')
        transaction.commit()
        registry = queryUtility(IRegistry)
        self.settings = registry.forInterface(IVideoConfiguration)
        # generate a set of mockup files:
        
        self.td = tempfile.mkdtemp(prefix = "on.video-tests.")
        for i in range(1, 6):
            sampledata = os.path.join(os.path.dirname(__file__), 'sample_video_%d.metadata' % i)
            shutil.copy(sampledata, self.td)
        sampledata = os.path.join(os.path.dirname(__file__), 'thumb.png')
        shutil.copy(sampledata, self.td)
        videofiles = ('sample_video_1_big.ogv', 'sample_video_1_medium.ogv', 'sample_video_1.mp4',
                      'sample_video_2.mp4', 'sample_video_2.avi',
                      'sample_video_3.mp4', 'sample_video_3_medium.ogv',
                      )
        subdir = os.path.join(self.td, "subfolder")
        os.mkdir(subdir)
        sampledata = os.path.join(os.path.dirname(__file__), 'sample_video_3.metadata')
        shutil.copy(sampledata, subdir)
        for d in self.td, subdir:
            for video in videofiles:
                o = open(os.path.join(d, video), "wb")
                o.write("1")
                o.close()
        self.settings.fspath = unicode(self.td)


    def tearDown(self):
        """remove the temp stuff"""
        # delete our private content, too
        shutil.rmtree(self.td)


    def test_view_registration(self):
        """Test the availability of the gallery view."""
        app = self.layer['app']
        browser = Browser(app)
        portal = self.layer['portal']
        portalURL = portal.absolute_url()
        browser.handleErrors = False
        browser.open(portalURL + '/login_form')
        browser.getControl(name='__ac_name').value = TEST_USER_NAME
        browser.getControl(name='__ac_password').value = TEST_USER_PASSWORD
        browser.getControl(name='submit').click()
        self.failUnless("Log out" in browser.contents)
        browser.open(portalURL + '/testfolder')
        self.failUnless("Video Gallery" in browser.contents)


    def test_gallery_item_filter(self):
        """Test that only videos and folders are being included in the
           listing.
        """
        portal = self.layer['portal']
        folder = portal['testfolder']
        for i in range(1,5):
            folder.invokeFactory('on.video.Video', id='test%d' % i,director='someone', filename='sample_video_3' )
        folder.invokeFactory('Document', id='some_document')
        folder.setLayout('videogallery')
        transaction.commit()
        app = self.layer['app']
        portalURL = portal.absolute_url()
        browser = Browser(app)
        browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD))
        browser.open(portalURL + '/testfolder')
        self.failUnless('test1' in browser.contents)
        self.failIf('some_document' in browser.contents)

    def test_gallery_item_director(self):
        """Test that the director is being displayed."""
        portal = self.layer['portal']
        folder = portal['testfolder']
        for i in range(1,4):
            folder.invokeFactory('on.video.Video', id='test%d' % i,
                                 director='someone',
                                 filename='sample_video_3')
        folder.setLayout('videogallery')
        transaction.commit()
        app = self.layer['app']
        browser = Browser(app)
        portalURL = portal.absolute_url()
        browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD))
        browser.open(portalURL + '/testfolder')
        self.failIf('someone' not in browser.contents)

    def test_gallery_item_title_too_long(self):
        """Test that the title is being cut off when it is too long."""
        portal = self.layer['portal']
        folder = portal['testfolder']
        folder.invokeFactory('on.video.Video', id='test1',
                             director='someone',
                             title=u'0123456789012345678901234567890123456789012345678901234567890',
                             filename='sample_video_3')
        folder.setLayout('videogallery')
        transaction.commit()
        app = self.layer['app']
        browser = Browser(app)
        portalURL = portal.absolute_url()
        browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD))
        browser.open(portalURL + '/testfolder')
        self.failUnless('...</h4>' in browser.contents)

    def test_gallery_item_title_not_too_long(self):
        """Test that the title is not being cut off when it is short enough."""
        portal = self.layer['portal']
        folder = portal['testfolder']
        folder.invokeFactory('on.video.Video', id='test2',
                             director='someone',
                             title=u'01234567890123456789012345678901234567890123456789',
                             filename='sample_video_3')
        folder.setLayout('videogallery')
        transaction.commit()
        app = self.layer['app']
        browser = Browser(app)
        portalURL = portal.absolute_url()
        browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD))
        browser.open(portalURL + '/testfolder')
        self.failUnless('789</h4>' in browser.contents)

    def test_gallery_banner_image_excluded(self):
        """Test that the title is not being cut off when it is short enough."""
        portal = self.layer['portal']
        portal.invokeFactory("Folder", id='videos', title='Galleries')
        g = portal['videos']
        for i in range(3):
            g.invokeFactory("Folder", id='g%d' % i, title='Gallery #%d' % i)
            subgallery = g['g%d' % i]
            for j in range(3):
                subgallery.invokeFactory('on.video.Video', id='video%d-%d' % (i, j), title='Video %d-%d' % (i, j))
        g.invokeFactory('Image', id='bannerimage', title='Banner Image')
        g.setLayout('videogallery')
        transaction.commit()
        app = self.layer['app']
        browser = Browser(app)
        portalURL = portal.absolute_url()
        browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD))
        browser.open(portalURL + '/videos')
        self.failIf('bannerimage' in browser.contents)



# configuration for this product: external file system paths etc.

# (c) 2012 oeko.net
# c/o toni mueller <support@oeko.net>
# license: GPLv3


ON_VIDEO_FS_PATH = u'/tmp'

# BEWARE: URLs must not contain Unicode!
ON_VIDEO_URL = "http://localhost/"

PLAYER = "flowplayer-3.2.5.swf"

MAX_WIDTH = 750
DEFAULT_WIDTH = 720
DEFAULT_HEIGHT = 540

MIN_HEIGHT = 100
MAX_HEIGHT = 900



# (c) 2012 oeko.net
# c/o toni mueller <support@oeko.net>
# license: GPLv3


from datetime import datetime

from zope.component import queryUtility
from zope.component import adapts, getMultiAdapter

import unittest2 as unittest

from Products.CMFCore.utils import getToolByName
from plone.registry.interfaces import IRegistry

from plone.app.testing import TEST_USER_ID
#from plone.app.testing import TEST_USER_NAME
#from plone.app.testing import TEST_USER_PASSWORD

from plone.app.testing import setRoles

from on.video.testing import ON_VIDEO_INTEGRATION_TESTING

from on.video.configuration import IVideoConfiguration
from on.video import config

class TestOnVideoConfig(unittest.TestCase):
    """Test the configuration of the product."""

    layer = ON_VIDEO_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_video_config_registry(self):
        """Verify that the default values in the registry have been set."""
        registry = queryUtility(IRegistry)
        self.assertNotEqual(registry, None)
        settings = registry.forInterface(IVideoConfiguration)
        self.assertEqual(settings.fspath, config.ON_VIDEO_FS_PATH)
        self.assertEqual(settings.urlbase, config.ON_VIDEO_URL)

    def test_install_on_video(self):
        quickinstaller = getToolByName(self.portal, 'portal_quickinstaller')
        self.assertTrue(quickinstaller.isProductInstalled('on.video'))

    def test_video_controlpanel_view(self):
        """Can we see the control panel?"""
        view = getMultiAdapter((self.portal, self.portal.REQUEST), 
                               name="on-video-settings")
        view = view.__of__(self.portal)
        configlet = view()
        self.failUnless('name="form.widgets.urlbase"' in configlet)
        self.failUnless('name="form.widgets.fspath"' in configlet)

    def test_video_control_panel_protected(self):
        """Can only we access the control panel?"""
        from AccessControl import Unauthorized
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.assertRaises(Unauthorized,
                          self.portal.restrictedTraverse,
                          '@@on-video-settings')

    def test_video_control_panel_read_url(self):
        """Retrieve the URL from the control panel"""
        registry = queryUtility(IRegistry)
        urlbase = registry.records[
            'on.video.configuration.IVideoConfiguration.urlbase']
        self.failUnless('urlbase' in IVideoConfiguration)
        self.assertEquals(urlbase.value, config.ON_VIDEO_URL)

    def test_video_control_panel_read_fs(self):
        """Retrieve the file system path from the control panel"""
        registry = queryUtility(IRegistry)
        fspath = registry.records[
            'on.video.configuration.IVideoConfiguration.fspath']
        self.failUnless('fspath' in IVideoConfiguration)
        self.assertEquals(fspath.value, config.ON_VIDEO_FS_PATH)




from setuptools import setup, find_packages
import os

version_file = os.path.join('on', 'video', 'version.txt')
version = open(version_file).read().strip()

setup(name='on.video',
      version=version,
      description="dynamic display of video files",
      long_description=open("README.rst").read() + \
                       open("CHANGES.rst").read(),
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Topic :: Multimedia :: Video",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='plone video dexterity',
      author='oeko.net mueller & brandt gbr',
      author_email='support@oeko.net',
      url='http://www.oeko.net/',
      license='GPLv3',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['on'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'Products.CMFPlone',
          'plone.app.dexterity [grok]',
          'plone.app.referenceablebehavior',
          'plone.app.relationfield',
          'collective.autopermission',
          'plone.app.registry',
          #'plone.namedfile[blobs]',
          #'gdata',
          # -*- Extra requirements: -*-
      ],
      extras_require = {
          'test': [
              'plone.app.testing',
          ]
      },
      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      #setup_requires=["PasteScript"],
      #paster_plugins = ["ZopeSkel"],
      )
